-- Version  : $Id: sesam_db_pg.sql,v 1.228 2012/01/04 13:59:51 sesam Exp $
--
-- Title    : Database for SEP sesam
-- File     : sesam_db_pg.sql
-- Author   : P.Schlegel / E.Einwanger  SEP AG D83629 Weyarn
-- Date     : 2007-05-01
-- 
--
-- ########## DROP TABLES ########################

-- ----------------------------------------------------------------------------
-- Tables without active & passive references
-- ----------------------------------------------------------------------------

drop table accounts;
drop table cal_sheets;
drop table defaults;
drop table sessions;
drop table users;
drop table profiles;
drop table preferences;
drop table notifications;
drop table media_results;

-- ----------------------------------------------------------------------------
-- Tables with only active references ( they refer to another table )
-- ----------------------------------------------------------------------------

drop table terms;
drop table capacities;
drop table drive_vendors;
drop table syntaxes;   			
drop table loader_contents;
drop table restore_users;
drop table result_lbls;                 
drop table task_group_relations;        
drop table servers;
drop table user_scheds;
drop table loader_devices;

-- ----------------------------------------------------------------------------
-- Tables with active and passive references
-- the sequence of drops MUST be kept due to inner relations
-- ----------------------------------------------------------------------------

drop table task_events;
drop table newday_events;
drop table mediapools_events;
drop table media_events;
drop table restore_events;
drop table command_events;
drop table migration_events;
drop table tasks;
drop table mediapool_relations;
drop table media;
drop table media_pools;
drop table restore_tasks;
drop table migration_tasks;
drop table results;
drop table task_sources;        
drop table hw_drives;
drop table hw_loaders;
drop table interfaces;
drop table clients;
drop table locations;

-- ----------------------------------------------------------------------------
-- Tables with only passive references ( they are referred from other tables )
-- ----------------------------------------------------------------------------

drop table commands;
drop table schedules;
drop table data_stores;
drop table media_actions;
drop table drive_types;
drop table media_types;
drop table mediapool_locations;
drop table oper_systems;
drop table drive_groups;
drop table restore_results;
drop table migration_results;
drop table task_groups;
drop table task_types;
drop table params_fix;


-- ########## CREATE TABLES ########################

-- ----------------------------------------------------------------------------
-- Tables with only passive references ( they are referred from other tables )
-- ----------------------------------------------------------------------------

CREATE TABLE drive_types (
	drive_type       VARCHAR(15) NOT NULL PRIMARY KEY,
	generic_type     VARCHAR(10) NOT NULL 
);

CREATE TABLE media_types (
	media_type       VARCHAR(20) NOT NULL PRIMARY KEY
);

CREATE TABLE oper_systems (
        os              VARCHAR(32) NOT NULL PRIMARY KEY,
        platform        VARCHAR(20) NOT NULL
); 

CREATE TABLE drive_groups (
	grp_id          INTEGER NOT NULL PRIMARY KEY,
	grp_name        VARCHAR(50),
	sms_flag        VARCHAR(1),
        restore_drive   INTEGER,
	text            VARCHAR(100),
	default_i_name	VARCHAR(64)
);

CREATE TABLE restore_results (
        restore_id      VARCHAR(64) NOT NULL PRIMARY KEY,
        start_time      VARCHAR(21) NOT NULL,
	r_task		VARCHAR(49) NOT NULL,
        sesam_date      VARCHAR(21),
        saveset         VARCHAR(64),
        task            VARCHAR(32),
        saveset_date    VARCHAR(21),
        cnt             INTEGER,
        restore_type    VARCHAR(2),
        genmode         VARCHAR(2),
        listmode        VARCHAR(32),
        map_mode        VARCHAR(1),
        tree_type       VARCHAR(2),
        mode            VARCHAR(2),
        recover         VARCHAR(64),
        onlinemode      VARCHAR(64),
        client_id       INTEGER,
        client          VARCHAR(64),
        target          VARCHAR(1024),
        user_name       VARCHAR(255),
	subtask_flag	CHAR(1),
	parent_task	VARCHAR(49),
        location        VARCHAR(50),
        drive_num       INTEGER,
        i_name          VARCHAR(64),
        filter          VARCHAR(255),
        reloc_source    VARCHAR(1024),
        state           VARCHAR(1) NOT NULL,
        stop_time       VARCHAR(21),
        pid             BIGINT,
        size            BIGINT,
        options         VARCHAR(255),
	throughput      VARCHAR(64),
        msg             VARCHAR(255),
	saveset_exist   VARCHAR(1)
);

CREATE TABLE migration_results (
        migration_id    VARCHAR(64) NOT NULL PRIMARY KEY,
	migration_task	VARCHAR(49) NOT NULL,
        sesam_date      VARCHAR(21),
        start_time      VARCHAR(21),
        stop_time       VARCHAR(21),
        source_pool     VARCHAR(32),
        target_pool     VARCHAR(32),
        source_drive    INTEGER,
        target_drive    INTEGER,
        from_date       VARCHAR(21),
        to_date         VARCHAR(21),
        saveset         VARCHAR(64),
        grpflag         VARCHAR(2),
        task            VARCHAR(32),
        saveset_date    VARCHAR(21),
        migrated_flag   VARCHAR(5),
        saveset_cnt     VARCHAR(4),
        backup_state    VARCHAR(32),
        cfdi_type       VARCHAR(16),
        client          VARCHAR,
        client_id       INTEGER,
        label           VARCHAR(30),
        filter          VARCHAR(255),
        options         VARCHAR(255),
        genmode         VARCHAR(2),
        listmode        VARCHAR(32),
        eol             INTEGER,
        i_name          VARCHAR(64),
	subtask_flag	VARCHAR(2),
	parent_task	VARCHAR(49),
        user_name       VARCHAR(255),
        user_comment    VARCHAR(255),
        pid             BIGINT,
        state           VARCHAR(1) NOT NULL,
        size            BIGINT,
	throughput      VARCHAR(64),
        msg             VARCHAR(255),
        migration_cmd   VARCHAR(64)
);

CREATE TABLE task_groups (
        grp_name        VARCHAR(50) NOT NULL PRIMARY KEY,
        text            VARCHAR(100),
        results_sts     VARCHAR(1),
        prepost         VARCHAR(16),
        prepost_switch  VARCHAR(1)	
);

CREATE TABLE task_types (
        backup_type       VARCHAR(32) NOT NULL PRIMARY KEY,
        backup_cmd        VARCHAR(20),
        backup_options    VARCHAR(128),
        prepost           VARCHAR(16),
        prepost_cmd       VARCHAR(20),
        restore_cmd       VARCHAR(20),
        restore_options   VARCHAR(128),
        r_prepost         VARCHAR(16),
        r_prepost_cmd     VARCHAR(20),
        source_prefix     VARCHAR(64),
        platform          VARCHAR(20),
        proxy             VARCHAR(20),
        login_flag        VARCHAR(1),
        flags             VARCHAR(32),
        protection        VARCHAR(4),
        comment           VARCHAR(40),
        interpreter       VARCHAR(16),
        restore_extern    VARCHAR(1),
        external_tool     VARCHAR(40),
        restore_as_path   VARCHAR(1),
        restore_selective VARCHAR(1),
        restore_plain     VARCHAR(1),
        restore_reloc     VARCHAR(1),
        restore_recover   VARCHAR(1),
        restore_online    VARCHAR(1),
        restore_start     VARCHAR(1),
        backup_online     VARCHAR(1),
        missing_cfdi      VARCHAR(4)
);

CREATE TABLE params_fix (
	customer        VARCHAR(50) NOT NULL PRIMARY KEY,
	rest_eol        INTEGER NOT NULL,
	min_free        INTEGER NOT NULL,
        type            VARCHAR(20) NOT NULL,
        version         VARCHAR(32) NOT NULL,
        inst_date       VARCHAR(21) NOT NULL,
        inst_cryp       VARCHAR(21),
        update_date     VARCHAR(21) NOT NULL,
        val1            VARCHAR(128),
        val2            VARCHAR(128),
        val3            VARCHAR(128),
        val4            VARCHAR(128),
        serialnumber    VARCHAR(128),
        server_id       VARCHAR(128),
	sts             VARCHAR(255),
	msg             VARCHAR(255)
);

CREATE TABLE mediapool_locations (
    name                VARCHAR(32) NOT NULL PRIMARY KEY,
    path                VARCHAR(1024),
    comment             VARCHAR(80),
    capacity            BIGINT,
    low_water_mark      BIGINT,
    high_water_mark     BIGINT,
    load_cmd            VARCHAR(255),
    unload_cmd          VARCHAR(255),
    access_mode         VARCHAR(255),
    status              VARCHAR(255),
    device_path         VARCHAR(255)
);

CREATE TABLE data_stores (
    name                VARCHAR(32) NOT NULL PRIMARY KEY,
    comment             VARCHAR(255),
    msg                 VARCHAR(1024),
    capacity            FLOAT,
    low_water_mark      FLOAT,
    high_water_mark     FLOAT,
    filled              FLOAT,
    total               FLOAT,
    used                FLOAT,
    free                FLOAT,
    last_action         VARCHAR(255),
    timestamp           VARCHAR(21),
    access_mode         VARCHAR(255),
    status              VARCHAR(255)
);

CREATE TABLE commands (
        name           VARCHAR(50) NOT NULL PRIMARY KEY,
        owner          VARCHAR(30) NOT NULL,
        type           VARCHAR(1)  NOT NULL,
        host           VARCHAR(64),
        user_name      VARCHAR(255),
        command        VARCHAR(255) NOT NULL,
        options        VARCHAR(255)
);

CREATE TABLE media_actions (
        name            VARCHAR(64) NOT NULL PRIMARY KEY,
        type            VARCHAR(2)  NOT NULL,
        owner           VARCHAR(30),
        action          VARCHAR(30),
        data_store      VARCHAR(32),
        pool            VARCHAR(16),
        label           VARCHAR(1024),
        drive           VARCHAR(16),
        suppress        VARCHAR(1),
        low_water_mark  BIGINT,
        high_water_mark BIGINT,
        check_flag      VARCHAR(2)
);



-- ----------------------------------------------------------------------------
-- Tables with active and passive references
-- the sequence of CREATE MUST be kept due to inner relations
-- ----------------------------------------------------------------------------

CREATE TABLE schedules (
	name            VARCHAR(30) NOT NULL PRIMARY KEY,
	cyc_flag        VARCHAR(1) NOT NULL,
	abs_flag        VARCHAR(1) NOT NULL,
	p_count         INTEGER,
	p_base          VARCHAR(3),
	mo              VARCHAR(1),
	tu              VARCHAR(1),
	we              VARCHAR(1),
	th              VARCHAR(1),
	fr              VARCHAR(1),
	sa              VARCHAR(1),
	su              VARCHAR(1),
	day_of_mth      VARCHAR(2),
	mth_of_yea      VARCHAR(2),
	wk_of_mth       VARCHAR(2),
	day_of_wk       VARCHAR(10),
	start_date      VARCHAR(21),
	end_date        VARCHAR(21),
	exec            VARCHAR(1),
	start_time      VARCHAR(5),
	duration        VARCHAR(6),
	day_offset      INTEGER,
        sepcomment      VARCHAR(64),
        comment         VARCHAR(64)
);

CREATE TABLE task_sources (
        task   	        VARCHAR(32)  NOT NULL,
        version_id      INTEGER      NOT NULL,
        source          VARCHAR(255) NOT NULL,
        timestamp       VARCHAR(21),
        user_name       VARCHAR(255),
        PRIMARY KEY (task,version_id)
);

CREATE TABLE results (
	saveset          VARCHAR(64) NOT NULL PRIMARY KEY,
        task             VARCHAR(32) NOT NULL,
	sesam_date       VARCHAR(21) NOT NULL,
        cnt              INTEGER NOT NULL ,
	fdi_type         VARCHAR(2),
	
	fdi_type_set     VARCHAR(2),
        media_pool       VARCHAR(16),
	lbl_cnt          INTEGER,
	drive_num        INTEGER,
	drive_num_set    INTEGER,
	
        i_name           VARCHAR(64),
	client_id        INTEGER,
	client           VARCHAR(64),
	client_os        VARCHAR(32),
	sesam_version    VARCHAR(64),
	
	sbc_version      VARCHAR(64),
	sms_version      VARCHAR(16),
	location         VARCHAR(50),
	source           VARCHAR(1024),
	exclude          VARCHAR(1024),
	
	exclude_type     VARCHAR(16),
        version_id       INTEGER,
        priority         INTEGER,
	locked           VARCHAR(1),
	eol              VARCHAR(21),
	start_time       VARCHAR(21),
	stop_time        VARCHAR(21),
	
        session_id       VARCHAR(32),
	state            VARCHAR(1),
	blocks           BIGINT,
	data_size        BIGINT,
	msg              VARCHAR(254),
	
        sms_flag         VARCHAR(1),
	sbc_start        VARCHAR(21),
	saveset_exist    VARCHAR(2),
        backup_cmd       VARCHAR(20),
        backup_type      VARCHAR(32),
        
        interpreter      VARCHAR(32),
        backup_options   VARCHAR(255),
        restore_options  VARCHAR(255),
        bsr_flag         VARCHAR(1),
        extern_flag      VARCHAR(2),
        online_flag      VARCHAR(2),
        
        compress_flag    VARCHAR(1),
        crypt_flag       VARCHAR(1),
        crypt_key        VARCHAR(128),
        verify_state     VARCHAR(2),
        verify_date      VARCHAR(21),
        
	original_saveset VARCHAR(80),
	copy_from        VARCHAR(80),
	reference_type   VARCHAR(8),
	reference_ssid   VARCHAR(80),
	based_on_full    VARCHAR(40),
	based_on         VARCHAR(40),
	throughput       VARCHAR(64),
	
        processed        BIGINT,
        notprocessed     BIGINT, 
        excluded         BIGINT, 
        user_comment     VARCHAR(256),
        uuid             VARCHAR(40)
);
CREATE INDEX results_task ON results (task);
CREATE INDEX results_start_time ON results (start_time);
CREATE INDEX results_state ON results (state);


CREATE TABLE restore_tasks (
	r_task		VARCHAR(49) NOT NULL PRIMARY KEY,
        saveset         VARCHAR(64) NOT NULL,
        task            VARCHAR(32) NOT NULL,
        saveset_date    VARCHAR(21) NOT NULL,
        cnt             INTEGER NOT NULL,
        restore_type    VARCHAR(2) NOT NULL,
        genmode         VARCHAR(2),
        listmode        VARCHAR(32),
        map_mode        VARCHAR(1),
        tree_type       VARCHAR(2) NOT NULL,
        mode            VARCHAR(2) NOT NULL,
        overwrite       VARCHAR(8),
        rename          VARCHAR(16),
        original        VARCHAR(2),
        recover         VARCHAR(64),
        onlinemode      VARCHAR(64),
        client_id       INTEGER NOT NULL,
        client          VARCHAR(64) NOT NULL,
        target          VARCHAR(1024) NOT NULL,
        target_server   VARCHAR(64),
        target_store    VARCHAR(1024),
        user_name       VARCHAR(255) NOT NULL,
	subtask_flag	VARCHAR(1),
	path_flag	VARCHAR(2),
	dump_flag	VARCHAR(2),
	parent_task	VARCHAR(49),
        location        VARCHAR(50),
        drive_num       INTEGER,
        i_name          VARCHAR(64),
        filter          VARCHAR(255),
        reloc_source    VARCHAR(1024),
        r_prepost       VARCHAR(16),
	make_stamp	VARCHAR(21) NOT NULL,
        eol             INTEGER,
        restore_cmd     VARCHAR(64),
        options         VARCHAR(255),
        crypt_flag      VARCHAR(1),
        crypt_key       VARCHAR(128),
	comment		VARCHAR(255)
);

CREATE TABLE locations (
        id              INTEGER NOT NULL PRIMARY KEY,
	name            VARCHAR(50),
	parent          INTEGER,
	describe        VARCHAR(128),
	contact         VARCHAR(128),
	comment         VARCHAR(128)
);

CREATE TABLE clients (
	client_id       INTEGER NOT NULL PRIMARY KEY,
        name            VARCHAR(64) NOT NULL UNIQUE,
	location        INTEGER references locations (id),
	os              VARCHAR(32) NOT NULL references oper_systems (os) DEFERRABLE INITIALLY DEFERRED,
	net_prot        VARCHAR(10) NOT NULL ,
	mac_address     VARCHAR(64),
	accessmode      VARCHAR(10),
	comment         VARCHAR(255),
	user_comment    VARCHAR(255),
   	permit          VARCHAR(1),
   	wol_flag        VARCHAR(2),
        user_name       VARCHAR(255),
        password        VARCHAR(255),
	access_options  VARCHAR(255),
	stpd_options    VARCHAR(32),
	stpd_port       VARCHAR(32),
        sbc_version     VARCHAR(20),
	sesam_version   VARCHAR(64),
	version_number  BIGINT,
        hw_platform     VARCHAR(32),
        package         VARCHAR(128),
        cores           INTEGER,
        data_mover      VARCHAR(64),
        vm_host         VARCHAR(64),
        vm_name         VARCHAR(255),
        vm_server_type  VARCHAR(32),
        license_level   VARCHAR(32),
        sesam_passwd    VARCHAR(255)
);

CREATE TABLE hw_loaders (
	loader_num      INTEGER NOT NULL PRIMARY KEY,
	device          VARCHAR(254) NOT NULL,
	client_id       INTEGER NOT NULL references clients (client_id) DEFERRABLE INITIALLY DEFERRED,
	name            VARCHAR(32),
	loader_type     VARCHAR(32) NOT NULL,
	ctrl            VARCHAR(10) NOT NULL,
	slots           INTEGER NOT NULL,
        drives          INTEGER,
        ports           INTEGER,
        transport       INTEGER,
        first_slot      INTEGER,
        last_slot       INTEGER,
        autounload      VARCHAR(1),
        barcode         VARCHAR(1),
        load_cmd        VARCHAR(255),
        unload_cmd      VARCHAR(255)
);

CREATE TABLE hw_drives (
	drive_num       INTEGER NOT NULL PRIMARY KEY,
	device          VARCHAR(254) NOT NULL ,
	client_id       INTEGER NOT NULL references clients (client_id) DEFERRABLE INITIALLY DEFERRED,
	drive_type      VARCHAR(15) NOT NULL references drive_types (drive_type) DEFERRABLE INITIALLY DEFERRED,
        name            VARCHAR(32),
	loader_num      INTEGER references hw_loaders (loader_num) DEFERRABLE INITIALLY DEFERRED,
	compress        VARCHAR(1),
	sts             VARCHAR(132),
	msg             VARCHAR(132),
	occupy          VARCHAR(20),
	access_mode     VARCHAR(4),
	done            INTEGER,
	run             INTEGER,
	waiting         INTEGER,
       	throughput      VARCHAR(32),
        grp_id          INTEGER references drive_groups (grp_id) DEFERRABLE INITIALLY DEFERRED,
        sms_cnts        INTEGER,
       	label           VARCHAR(30),
        timeout         INTEGER,
        media_timeout   INTEGER,
        clean_bit       VARCHAR(1),
        loader_drive    INTEGER,
        path            VARCHAR(255),
        device_path     VARCHAR(255),
        data_store      VARCHAR(32) references data_stores (name) DEFERRABLE INITIALLY DEFERRED,
        eject_flag      VARCHAR(1),
        mount_cmd       VARCHAR(255),
        umount_cmd      VARCHAR(255),
        online_cmd      VARCHAR(255),
        load_cmd        VARCHAR(255),
        unload_cmd      VARCHAR(255),
        eject_cmd       VARCHAR(255),
        block_size      INTEGER,
        sms_nr          INTEGER,
        segment_size    INTEGER
);

CREATE TABLE migration_tasks (
        name            VARCHAR(49) NOT NULL PRIMARY KEY,
        source_pool     VARCHAR(32),
        target_pool     VARCHAR(32),
        source_drive    INTEGER references hw_drives (drive_num) DEFERRABLE INITIALLY DEFERRED,
        target_drive    INTEGER,
        sesam_date      VARCHAR(21),
        saveset_cnt     INTEGER,
        date_start      VARCHAR(21),
        date_end        VARCHAR(21),
        absolute_flag   VARCHAR(2),
        saveset         VARCHAR(64),
        grpflag         VARCHAR(2),
        task            VARCHAR(32),
        task_group      VARCHAR(32),
        backup_state    VARCHAR(32),
        cfdi_type       VARCHAR(16),
        label           VARCHAR(30),
        genmode         VARCHAR(2),
        migrated_flag   VARCHAR(5),
        original        VARCHAR(2),
        client          VARCHAR(64),
        i_name          VARCHAR(64),
        filter          VARCHAR(255),
        submit_flag     VARCHAR(2),
        make_stamp      VARCHAR(21),
        eol             INTEGER,
        migration_cmd   VARCHAR(64),
        options         VARCHAR(255),
        user_comment    VARCHAR(255)
);



CREATE TABLE media_pools (
        name                 VARCHAR(16) NOT NULL PRIMARY KEY,
        pool_id              INTEGER NOT NULL,
        descript             VARCHAR(80),
        drive_grp            INTEGER NOT NULL references drive_groups (grp_id) DEFERRABLE INITIALLY DEFERRED,
        media_strg           VARCHAR(64) NOT NULL,
        media_chg            VARCHAR(8) NOT NULL,
        close_on_init        VARCHAR(4),
        close_on_init_grp    VARCHAR(4),
        inactive             VARCHAR(2),
        disk_capac           VARCHAR(16),
        disk_dir             VARCHAR(254),
        eol                  INTEGER,
        accept_empty         VARCHAR(2),
        accept_eol           VARCHAR(2),
        accept_spare         VARCHAR(2),
        accept_other         VARCHAR(2),
        allow_move           VARCHAR(2),
        use_storagepools     VARCHAR(2),
        readcheck_limit      INTEGER,
        readcheck_repeatrate INTEGER,
        readcheck_overdue    INTEGER
);

CREATE TABLE mediapool_relations (
    mediapool           VARCHAR(16) references media_pools (name) DEFERRABLE INITIALLY DEFERRED,
    mediapool_location  VARCHAR(32) references mediapool_locations (name) DEFERRABLE INITIALLY DEFERRED,
    priority            INTEGER,
    comment             VARCHAR(80)
);


CREATE TABLE interfaces (
	client_id       INTEGER  NOT NULL references clients (client_id) DEFERRABLE INITIALLY DEFERRED,
        i_name          VARCHAR(64) NOT NULL UNIQUE,
        PRIMARY KEY (client_id,i_name)
);

CREATE TABLE loader_devices (
	loader_num      INTEGER  NOT NULL references hw_loaders (loader_num) DEFERRABLE INITIALLY DEFERRED,
        slot            INTEGER, 
        device_path     VARCHAR(1024)
);



-- task_events.object references tasks.task or task_groups.grp_name

CREATE TABLE task_events (
        id              INTEGER NOT NULL PRIMARY KEY,
        object          VARCHAR(50) NOT NULL,
        schedule        VARCHAR(30), 
        grp_flag        VARCHAR(1) NOT NULL,
        priority        INTEGER NOT NULL,
        suppress        VARCHAR(1),
        owner           VARCHAR(30) NOT NULL,
        fdi_type        VARCHAR(2) NOT NULL,
        media_pool      VARCHAR(16) NOT NULL,
        label           VARCHAR(30),
        drive_num       INTEGER references hw_drives (drive_num) DEFERRABLE INITIALLY DEFERRED,
	i_name		VARCHAR(64) references interfaces (i_name) DEFERRABLE INITIALLY DEFERRED,
        data_mover      VARCHAR(64),
        max_since_full  VARCHAR(21),
        migration_task  VARCHAR(32),
        reference_type  VARCHAR(8),
        reference_ssid  VARCHAR(80),
        online_flag     VARCHAR(2)
);

CREATE TABLE newday_events (
        id              INTEGER NOT NULL PRIMARY KEY,
        object          VARCHAR(50) NOT NULL references params_fix (customer) DEFERRABLE INITIALLY DEFERRED,
        schedule        VARCHAR(30),
        priority        INTEGER NOT NULL,
        suppress        VARCHAR(1),
        owner           VARCHAR(30) NOT NULL
);

CREATE TABLE media_events (
        id              INTEGER NOT NULL PRIMARY KEY,
        object          VARCHAR(16) NOT NULL,
        data_store      VARCHAR(32) references data_stores (name) DEFERRABLE INITIALLY DEFERRED,
        pool            VARCHAR(16) references media_pools (name) DEFERRABLE INITIALLY DEFERRED,
	loader          INTEGER     references hw_loaders (loader_num) DEFERRABLE INITIALLY DEFERRED,
        schedule        VARCHAR(30), 
        priority        INTEGER NOT NULL,
        suppress        VARCHAR(1),
        owner           VARCHAR(30) NOT NULL,
        type            VARCHAR(1),
        action          VARCHAR(30),
        drive           VARCHAR(16),
        label           VARCHAR(1024),
        grp_flag        VARCHAR(1),
        m_count         INTEGER,
	slot_range      VARCHAR(128),
        emergency       VARCHAR(30),
        init_flags      VARCHAR(32),
        check_flag      VARCHAR(2),
        media_type      VARCHAR(20),
        low_water_mark  BIGINT,
        high_water_mark BIGINT
);


CREATE TABLE mediapools_events (
        id              INTEGER NOT NULL PRIMARY KEY,
        object          VARCHAR(16) NOT NULL references media_pools (name) DEFERRABLE INITIALLY DEFERRED,
        schedule        VARCHAR(30), 
        action          VARCHAR(30),
        drive           VARCHAR(16),
        priority        INTEGER NOT NULL,
        suppress        VARCHAR(1),
        owner           VARCHAR(30) NOT NULL,
        grp_flag        VARCHAR(1) NOT NULL,
        m_count         INTEGER,
        label           VARCHAR(1024),
	loader          INTEGER,        -- refers to hw_loaders.loader_num
	slot_range      VARCHAR(128),
        emergency       VARCHAR(30),
        init_flags      VARCHAR(32),
        check_flag      VARCHAR(2),
        media_type      VARCHAR(20)
);

CREATE TABLE restore_events (
        id               INTEGER NOT NULL PRIMARY KEY, 
	object           VARCHAR(49) NOT NULL references restore_tasks (r_task) DEFERRABLE INITIALLY DEFERRED,
        schedule         VARCHAR(30),
        priority         INTEGER NOT NULL,
        suppress         VARCHAR(1),
        owner            VARCHAR(30) NOT NULL,
        drive_num        INTEGER,
        i_name           VARCHAR(64),
        listmode         VARCHAR(32),
        absolute_flag    VARCHAR(1),
        saveset_start    INTEGER,
        saveset_end      INTEGER,
        min_max          VARCHAR(21),
        date_start       VARCHAR(21),
        date_end         VARCHAR(21),
        state            VARCHAR(12),
        cfdi_type        VARCHAR(12),
        media_pools      VARCHAR(255),
        media_tolerance  VARCHAR(21),
        media_preference VARCHAR(21),
        data_mover       VARCHAR(64),
        use_saveset      VARCHAR(64),
        restore_cmd      VARCHAR(64),
        options          VARCHAR(255),
        reference_type   VARCHAR(8),
        reference_id     VARCHAR(80)
);

CREATE TABLE command_events (
        id               INTEGER NOT NULL PRIMARY KEY,
        object           VARCHAR(50) NOT NULL,
        schedule         VARCHAR(30),
        priority         INTEGER NOT NULL,
        suppress         VARCHAR(1),
        owner            VARCHAR(30) NOT NULL,
        type             VARCHAR(1) NOT NULL,
        host             VARCHAR(64),
        user_name        VARCHAR(255),
        command          VARCHAR(255) NOT NULL,
        options          VARCHAR(255)
);

CREATE TABLE migration_events (
        id               INTEGER NOT NULL PRIMARY KEY,
        object           VARCHAR(49) NOT NULL references migration_tasks (name) DEFERRABLE INITIALLY DEFERRED,
        schedule         VARCHAR(30),
        priority         INTEGER NOT NULL,
        suppress         VARCHAR(1),
        owner            VARCHAR(30),
        saveset          VARCHAR(64),
        grpflag          VARCHAR(2),
        task             VARCHAR(32),
        task_group       VARCHAR(32),
        saveset_cnt      INTEGER,
        target_drive     INTEGER,
        target_pool      VARCHAR(16),
        i_name           VARCHAR(64),
        migrated_flag    VARCHAR(5),
        absolute_flag    VARCHAR(2),
        date_start       VARCHAR(21),
        date_end         VARCHAR(21),
        cfdi_type        VARCHAR(12),
        state            VARCHAR(12),
        label            VARCHAR(30),
        listmode         VARCHAR(32),
        submit_flag      VARCHAR(2),
        visible          VARCHAR(1),
        data_mover       VARCHAR(64),
        migration_cmd    VARCHAR(64),
        user_comment     VARCHAR(128)
);

CREATE TABLE media (
    	label           VARCHAR(30) NOT NULL PRIMARY KEY,
    	barcode         VARCHAR(30) ,
        pool            VARCHAR(16) NOT NULL references media_pools (name) DEFERRABLE INITIALLY DEFERRED,
        id              INTEGER NOT NULL ,
	locked          VARCHAR(1) NOT NULL ,
	sesam_date      VARCHAR(21) NOT NULL ,
	eol             VARCHAR(21),
        eol_changedby   VARCHAR(32),
	drive_num       INTEGER,
        init_drive      INTEGER,
	sequence        INTEGER,
	media_type      VARCHAR(20) NOT NULL references media_types (media_type) DEFERRABLE INITIALLY DEFERRED,
	loader          INTEGER,
	slot            INTEGER,
	filled          BIGINT,
	eom_state       VARCHAR(1),
	first_init      VARCHAR(21),
        close_tape      VARCHAR(1),
	init_flag       VARCHAR(1),
	error           INTEGER,
	error_delt      INTEGER,
	use_count       INTEGER,
	location        VARCHAR(32),
	duplflag        VARCHAR(1),
	comment         VARCHAR(254),
        user_comment    VARCHAR(254),
        readcheck_state VARCHAR(1),
        readcheck_msg   VARCHAR(254)
);

CREATE TABLE tasks (
	task   	           VARCHAR(32) NOT NULL PRIMARY KEY,
        backup_type        VARCHAR(32) references task_types (backup_type) DEFERRABLE INITIALLY DEFERRED,
	client_id          INTEGER NOT NULL references clients (client_id) DEFERRABLE INITIALLY DEFERRED,
	source             VARCHAR(1024),
	exclude            VARCHAR(1024),
	comment            VARCHAR(255),		
	results_sts        VARCHAR(1),
        backup_cmd         VARCHAR(64),
	backup_options     VARCHAR(255),
	restore_options    VARCHAR(255),
	prepost            VARCHAR(16),
        r_prepost          VARCHAR(16),
        nfs_mount          VARCHAR(1),
        multi_source       VARCHAR(2),
        multi_source_type  VARCHAR(512),
        filesystem         VARCHAR(1),
        bsr_flag           VARCHAR(1),
        compress_flag      VARCHAR(1),
        crypt_flag         VARCHAR(1),
        crypt_key          VARCHAR(128),
        crypt_savekey_flag VARCHAR(2),
        snapshot_flags     VARCHAR(64),
        data_mover         VARCHAR(64),
        user_name          VARCHAR(255),
        password           VARCHAR(255)
);

-- ----------------------------------------------------------------------------
-- Tables with only active references ( they refer to another table )
-- ----------------------------------------------------------------------------

CREATE TABLE terms (
       id               INTEGER NOT NULL PRIMARY KEY,
       type             VARCHAR(1) NOT NULL,
       schedule         VARCHAR(30) NOT NULL references schedules (name) DEFERRABLE INITIALLY DEFERRED,
       next_exec        VARCHAR(21),
       next_end         VARCHAR(21)
);

CREATE TABLE capacities (
	drive_type       VARCHAR(15) NOT NULL references drive_types (drive_type) DEFERRABLE INITIALLY DEFERRED,
	media_type       VARCHAR(20) NOT NULL references media_types (media_type) DEFERRABLE INITIALLY DEFERRED,
	size             BIGINT NOT NULL,
        protection       VARCHAR(4) NOT NULL,
        PRIMARY KEY (drive_type,media_type)
);

CREATE TABLE drive_vendors (
	drive_type       VARCHAR(15) NOT NULL references drive_types (drive_type) DEFERRABLE INITIALLY DEFERRED,
	vendor_type      VARCHAR(16) NOT NULL,
        protection      VARCHAR(4) NOT NULL,
	PRIMARY KEY (drive_type,vendor_type)
);

CREATE TABLE syntaxes (
	os              VARCHAR(32) NOT NULL references oper_systems (os) DEFERRABLE INITIALLY DEFERRED,
	command         VARCHAR(32) NOT NULL,
	syntax          VARCHAR(64) NOT NULL,
        PRIMARY KEY (os,command)
);

CREATE TABLE task_group_relations (
        grp_name        VARCHAR(50) NOT NULL references task_groups (grp_name) DEFERRABLE INITIALLY DEFERRED,
	task   	        VARCHAR(32) NOT NULL references tasks (task) DEFERRABLE INITIALLY DEFERRED,
	task_order      INTEGER,
        PRIMARY KEY (grp_name,task)
);

CREATE TABLE result_lbls (
        saveset         VARCHAR(64) NOT NULL references results (saveset) DEFERRABLE INITIALLY DEFERRED,
	num             INTEGER NOT NULL,
	label           VARCHAR(30) NOT NULL,
        segment         VARCHAR(20) NOT NULL,
        PRIMARY KEY (saveset,num)
);

CREATE TABLE restore_users (
	name            VARCHAR(255) NOT NULL,
	task            VARCHAR(32) NOT NULL references tasks (task) DEFERRABLE INITIALLY DEFERRED,
	path            VARCHAR(132) NOT NULL,
	comment         VARCHAR(100),
        PRIMARY KEY (name,task,path)
);

CREATE TABLE loader_contents (
        loader          INTEGER	NOT NULL references hw_loaders (loader_num) DEFERRABLE INITIALLY DEFERRED,
        object          VARCHAR(10) NOT NULL,
        number          INTEGER	NOT NULL,
        lbl             VARCHAR(30),
	barcode         VARCHAR(30),
        check_lbl       VARCHAR(1),
        checkedlbl      VARCHAR(30),
        from_object     VARCHAR(10),
        from_number     INTEGER,
        to_object       VARCHAR(10),
        to_number       INTEGER,	
	location        VARCHAR(32),
        PRIMARY KEY (loader,object,number)
);

CREATE TABLE servers (
        name            VARCHAR(64) NOT NULL PRIMARY KEY,
	os              VARCHAR(32) NOT NULL references oper_systems (os) DEFERRABLE INITIALLY DEFERRED,
        ip_address      VARCHAR(64),
        rmi_port        INTEGER,
        ctrl_port       INTEGER,
        db_port         INTEGER,
        db_name         VARCHAR(73),
        db_alias        VARCHAR(73),
        db_type         VARCHAR(32),
        connect         VARCHAR(1),
        comment         VARCHAR(80),
        id              INTEGER
);

CREATE TABLE user_scheds (
     name            VARCHAR(30) NOT NULL references schedules (name) DEFERRABLE INITIALLY DEFERRED,
     date_name       VARCHAR(32) NOT NULL,
     date_pattern    VARCHAR(32) NOT NULL,
     next_exec       VARCHAR(32),
     notices         VARCHAR(100),
     PRIMARY KEY (name,date_name)
);

-- ----------------------------------------------------------------------------
-- Tables without active & passive references
-- ----------------------------------------------------------------------------

CREATE TABLE accounts (
        account_name       VARCHAR(64) NOT NULL PRIMARY KEY,
        customer           VARCHAR(64),
        smtp_email_address VARCHAR(64) NOT NULL,
        smtp_server        VARCHAR(64) NOT NULL,
        smtp_port          INTEGER,
        ssl_options        VARCHAR(32),
        mail_username      VARCHAR(64),
        mail_password      VARCHAR(128),
        mail_to            VARCHAR(128) NOT NULL,
        mail_cc            VARCHAR(128),
        mail_bcc           VARCHAR(128),
        service_location   VARCHAR(64)
);

CREATE TABLE cal_sheets (
	id		VARCHAR(64) PRIMARY KEY,
	start		VARCHAR(21),
	end_time        VARCHAR(21),
	ev_type		VARCHAR(3),
	ev_id		INTEGER,
	schedule	VARCHAR(30),
        priority        INTEGER,
	state		VARCHAR(3),
        object          VARCHAR(132),
        suppress        VARCHAR(1),
	msg		VARCHAR(255)
);

CREATE TABLE defaults (	
        key             VARCHAR(32) NOT NULL,
        user_name       VARCHAR(255) NOT NULL ,
        value           VARCHAR(4096) NOT NULL,
        host            VARCHAR(255),
        protection      VARCHAR(4),
        PRIMARY KEY     (key,user_name)
);

CREATE TABLE sessions (
	session_id      VARCHAR(32) PRIMARY KEY,
	object		VARCHAR(32),
	comment		VARCHAR(80)
);

CREATE TABLE preferences (
        pref_node       VARCHAR(255)  NOT NULL,
        pref_user       VARCHAR(255)  NOT NULL,
        pref_key        VARCHAR(255)  NOT NULL,
        pref_value      VARCHAR(8192) NOT NULL,
        profile_name    VARCHAR(64),
        PRIMARY KEY     (pref_node,pref_user,pref_key)
);

CREATE TABLE profiles (
        name            VARCHAR(64)   NOT NULL,
        node            VARCHAR(255)  NOT NULL,
        user_name       VARCHAR(255)  NOT NULL,
        key             VARCHAR(255)  NOT NULL,
        value           VARCHAR(8192) NOT NULL,
        PRIMARY KEY     (name,node,user_name,key)
);

CREATE TABLE notifications (
        id              BIGINT NOT NULL PRIMARY KEY,
        module          VARCHAR(64) NOT NULL,
	sesam_date      VARCHAR(21) NOT NULL,
	timestamp       VARCHAR(21) NOT NULL,
        severity        INTEGER NOT NULL,
        object_type     VARCHAR(32),
        object          VARCHAR(64),
        action          VARCHAR(30),
	state           VARCHAR(1),
	acknowledged    VARCHAR(32),
	ack_by          VARCHAR(64),
	ack_date        VARCHAR(21),
        pid             BIGINT,
	message_id      VARCHAR(32),
	message         VARCHAR(512)
);


CREATE TABLE media_results (
        id              VARCHAR(64) NOT NULL PRIMARY KEY,
        action          VARCHAR(30) NOT NULL,
	sesam_date      VARCHAR(21) NOT NULL,
        run             INTEGER NOT NULL ,
        priority        INTEGER,
	state           VARCHAR(1),
	check_flag      VARCHAR(2),
        data_store      VARCHAR(32),    -- fetched from data_store.name
        pool            VARCHAR(16),    -- fetched from media_pools.name
        label           VARCHAR(30),    -- fetched from media.label
    	barcode         VARCHAR(30),    -- fetched from media.barcode
	client_id       INTEGER,        -- fetched from clients.client_id
	client          VARCHAR(64),    -- fetched from clients.name
        drive_num       INTEGER,        -- fetched from hw_drives.drive_num
        device          VARCHAR(254),   -- fetched from hw_drives.device
        device_name     VARCHAR(254),   -- fetched from hw_drives.name
	start_time      VARCHAR(21),
	stop_time       VARCHAR(21),
        pid             BIGINT,
        session_id      VARCHAR(32),    -- refers to media_results.id (session)
	loader          INTEGER,        -- refers to hw_loaders.loader_num
	slot_range      VARCHAR(128),
	blocks          BIGINT,
	size            BIGINT,
	throughput      VARCHAR(64),
	drive_type      VARCHAR(15),    -- refers to drive_types.drive_type
	media_type      VARCHAR(20),    -- refers to media_types.media_type
	message         VARCHAR(254)
);
CREATE INDEX media_results_label ON media_results (label);


CREATE TABLE users (
        user_name VARCHAR(255) not null,
        credential VARCHAR(128),
        client VARCHAR(64),
        active VARCHAR(5),
        permission VARCHAR(10),
        alias VARCHAR(64),
        PRIMARY KEY (user_name,client)
);
